@@ -1,4 +1,3 @@
-
 //! mpmc synchronized channel implementation
 //! support an unbounded mpmc queue, senders would not block
 //! receivers would block if there is no data until sender send data
@@ -21,7 +20,11 @@ use crossbeam::queue::SegQueue;
 struct InnerQueue<T> {
     queue: SegQueue<T>,
     // thread/coroutine for wake up
-    sem: Semphore,
+    wake_recv: Semphore,
+    // thread/coroutine for wake up
+    wake_sender: Semphore,
+    // chan buffer length
+    buffer_len: usize,
     // The number of tx channels which are currently using this queue.
     tx_ports: AtomicUsize,
     // if rx is dropped
@@ -29,10 +32,19 @@ struct InnerQueue<T> {
 }
 
 impl<T> InnerQueue<T> {
+
+    /// default is an Unbounded buffer
     pub fn new() -> InnerQueue<T> {
+        InnerQueue::new_buffer(usize::MAX)
+    }
+
+    /// have buffer channel. If the buffered message exceeds the limit, the sender blocks until the message is consumed
+    pub fn new_buffer(mut buffer: usize) -> InnerQueue<T> {
         InnerQueue {
             queue: SegQueue::new(),
-            sem: Semphore::new(0),
+            wake_recv: Semphore::new(0),
+            wake_sender: Semphore::new(0),
+            buffer_len: buffer,
             tx_ports: AtomicUsize::new(1),
             rx_ports: AtomicUsize::new(1),
         }
@@ -42,12 +54,21 @@ impl<T> InnerQueue<T> {
         if self.rx_ports.load(Ordering::Acquire) == 0 {
             return Err(SendError(t));
         }
-
         self.queue.push(t);
-        self.sem.post();
+        self.wake_recv.post();
+        if self.queue.len() >= self.buffer_len {
+            self.wake_sender.wait();
+        }
         Ok(())
     }
 
+    #[inline]
+    fn wake_sender(&self) {
+        if self.queue.len() >= self.buffer_len {
+            self.wake_sender.post();
+        }
+    }
+
     pub fn recv(&self, dur: Option<Duration>) -> Result<T, RecvTimeoutError> {
         match self.try_recv() {
             Ok(data) => return Ok(data),
@@ -56,16 +77,19 @@ impl<T> InnerQueue<T> {
         }
 
         match dur {
-            None => self.sem.wait(),
+            None => self.wake_recv.wait(),
             Some(t) => {
-                if !self.sem.wait_timeout(t) {
+                if !self.wake_recv.wait_timeout(t) {
                     return Err(RecvTimeoutError::Timeout);
                 }
             }
         }
 
         match self.queue.pop() {
-            Some(data) => Ok(data),
+            Some(data) => {
+                self.wake_sender();
+                Ok(data)
+            }
             None => match self.tx_ports.load(Ordering::Acquire) {
                 0 => Err(RecvTimeoutError::Disconnected),
                 _n => unreachable!("mpmc recv found no data"),
@@ -74,7 +98,7 @@ impl<T> InnerQueue<T> {
     }
 
     pub fn try_recv(&self) -> Result<T, TryRecvError> {
-        if !self.sem.try_wait() {
+        if !self.wake_recv.try_wait() {
             return match self.tx_ports.load(Ordering::Acquire) {
                 0 => Err(TryRecvError::Disconnected),
                 _ => Err(TryRecvError::Empty),
@@ -82,7 +106,10 @@ impl<T> InnerQueue<T> {
         }
 
         match self.queue.pop() {
-            Some(data) => Ok(data),
+            Some(data) => {
+                self.wake_sender();
+                Ok(data)
+            }
             None => match self.tx_ports.load(Ordering::Acquire) {
                 0 => Err(TryRecvError::Disconnected),
                 _ => unreachable!("mpmc try_recv found no data"),
@@ -99,8 +126,8 @@ impl<T> InnerQueue<T> {
             1 => {
                 // there is no tx port any more
                 // should tell all the waited rx to come back
-                while self.sem.get_value() == 0 {
-                    self.sem.post();
+                while self.wake_recv.get_value() == 0 {
+                    self.wake_recv.post();
                 }
             }
             n if n > 1 => {}
@@ -177,7 +204,7 @@ impl<T> Sender<T> {
 
     /// return how many elements in the queue that are not consumed by receivers
     pub fn pressure(&self) -> usize {
-        self.inner.sem.get_value()
+        self.inner.wake_recv.get_value()
     }
 }
 
@@ -546,7 +573,7 @@ mod tests {
             drop(tx);
             rx.recv().unwrap();
         })
-        .join();
+            .join();
         // What is our res?
         assert!(res.is_err());
     }
@@ -627,7 +654,7 @@ mod tests {
         let res = thread::spawn(move || {
             assert!(*rx.recv().unwrap() == 10);
         })
-        .join();
+            .join();
         assert!(res.is_err());
     }
 
@@ -652,7 +679,7 @@ mod tests {
             let _ = thread::spawn(move || {
                 tx.send(1).unwrap();
             })
-            .join();
+                .join();
         }
     }
 
@@ -664,7 +691,7 @@ mod tests {
                 let res = thread::spawn(move || {
                     rx.recv().unwrap();
                 })
-                .join();
+                    .join();
                 assert!(res.is_err());
             });
             let _t = thread::spawn(move || {
