@@ -1,4 +1,3 @@
-
 //! compatible with std::sync::mpsc except for both thread and coroutine
 //! please ref the doc from std::sync::mpsc
 use std::fmt;
@@ -8,16 +7,22 @@ use std::sync::mpsc::{RecvError, RecvTimeoutError, SendError, TryRecvError};
 use std::sync::Arc;
 use std::time::{Duration, Instant};
 use crossbeam::queue::SegQueue;
+use crate::coroutine::yield_now;
+use crate::std::sync::Semphore;
 
 use super::{AtomicOption, Blocker};
-// TODO: SyncSender
+
 /// /////////////////////////////////////////////////////////////////////////////
 /// InnerQueue
 /// /////////////////////////////////////////////////////////////////////////////
 struct InnerQueue<T> {
     queue: SegQueue<T>,
     // thread/coroutine for wake up
-    to_wake: AtomicOption<Arc<Blocker>>,
+    wake_recv: AtomicOption<Arc<Blocker>>,
+    // thread/coroutine for wake up
+    wake_sender: Semphore,
+    // chan buffer length
+    buffer_len: usize,
     // The number of tx channels which are currently using this queue.
     channels: AtomicUsize,
     // if rx is dropped
@@ -25,10 +30,18 @@ struct InnerQueue<T> {
 }
 
 impl<T> InnerQueue<T> {
+    /// default is an Unbounded buffer
     pub fn new() -> InnerQueue<T> {
+        Self::new_buffer(usize::MAX)
+    }
+
+    /// have buffer channel. If the buffered message exceeds the limit, the sender blocks until the message is consumed
+    pub fn new_buffer(mut buf: usize) -> InnerQueue<T> {
         InnerQueue {
             queue: SegQueue::new(),
-            to_wake: AtomicOption::none(),
+            wake_recv: AtomicOption::none(),
+            wake_sender: Semphore::new(0),
+            buffer_len: buf,
             channels: AtomicUsize::new(1),
             port_dropped: AtomicBool::new(false),
         }
@@ -39,9 +52,12 @@ impl<T> InnerQueue<T> {
             return Err(t);
         }
         self.queue.push(t);
-        if let Some(w) = self.to_wake.take(Ordering::Acquire) {
+        if let Some(w) = self.wake_recv.take(Ordering::Acquire) {
             w.unpark();
         }
+        if self.queue.len() >= self.buffer_len {
+            self.wake_sender.wait();
+        }
         Ok(())
     }
 
@@ -53,7 +69,7 @@ impl<T> InnerQueue<T> {
 
         let cur = Blocker::current();
         // register the waiter
-        self.to_wake.swap(cur.clone(), Ordering::Release);
+        self.wake_recv.swap(cur.clone(), Ordering::Release);
         // re-check the queue
         match self.try_recv() {
             Err(TryRecvError::Empty) => {
@@ -61,7 +77,7 @@ impl<T> InnerQueue<T> {
             }
             data => {
                 // no need to park, contention with send
-                if let Some(w) = self.to_wake.take(Ordering::Acquire) {
+                if let Some(w) = self.wake_recv.take(Ordering::Acquire) {
                     w.unpark();
                 }
                 cur.park(dur).ok();
@@ -73,13 +89,31 @@ impl<T> InnerQueue<T> {
         self.try_recv()
     }
 
+    #[inline]
+    fn wake_sender(&self) {
+        if self.queue.len() >= self.buffer_len {
+            self.wake_sender.post();
+        }
+    }
+
     pub fn try_recv(&self) -> Result<T, TryRecvError> {
         match self.queue.pop() {
-            Some(data) => Ok(data),
+            Some(data) => {
+                self.wake_sender();
+                Ok(data)
+            }
             None => {
                 match self.channels.load(Ordering::Acquire) {
                     // there is no sender any more, should re-check
-                    0 => self.queue.pop().ok_or(TryRecvError::Disconnected),
+                    0 => {
+                        match self.queue.pop() {
+                            None => { Err(TryRecvError::Disconnected) }
+                            Some(v) => {
+                                self.wake_sender();
+                                Ok(v)
+                            }
+                        }
+                    }
                     _ => Err(TryRecvError::Empty),
                 }
             }
@@ -94,10 +128,11 @@ impl<T> InnerQueue<T> {
         match self.channels.fetch_sub(1, Ordering::AcqRel) {
             1 => {
                 self
-                    .to_wake
+                    .wake_recv
                     .take(Ordering::Relaxed)
                     .map(|w| w.unpark());
-            },
+                self.wake_sender();
+            }
             n if n > 1 => {}
             n => panic!("bad number of channels left {}", n),
         }
@@ -113,7 +148,7 @@ impl<T> InnerQueue<T> {
 impl<T> Drop for InnerQueue<T> {
     fn drop(&mut self) {
         assert_eq!(self.channels.load(Ordering::Acquire), 0);
-        assert!(self.to_wake.is_none());
+        assert!(self.wake_recv.is_none());
     }
 }
 
@@ -141,12 +176,18 @@ pub struct Sender<T> {
 }
 
 unsafe impl<T: Send> Send for Sender<T> {}
+
 // impl<T> !Sync for Sender<T> {}
 impl<T: Send> UnwindSafe for Sender<T> {}
+
 impl<T: Send> RefUnwindSafe for Sender<T> {}
 
 pub fn channel<T>() -> (Sender<T>, Receiver<T>) {
-    let a = Arc::new(InnerQueue::new());
+    channel_buf(usize::MAX)
+}
+
+pub fn channel_buf<T>(buf: usize) -> (Sender<T>, Receiver<T>) {
+    let a = Arc::new(InnerQueue::new_buffer(buf));
     (Sender::new(a.clone()), Receiver::new(a))
 }
 
@@ -546,7 +587,7 @@ mod tests {
             drop(tx);
             rx.recv().unwrap();
         })
-        .join();
+            .join();
         // What is our res?
         assert!(res.is_err());
     }
@@ -627,7 +668,7 @@ mod tests {
         let res = thread::spawn(move || {
             assert!(*rx.recv().unwrap() == 10);
         })
-        .join();
+            .join();
         assert!(res.is_err());
     }
 
@@ -652,7 +693,7 @@ mod tests {
             let _ = thread::spawn(move || {
                 tx.send(1).unwrap();
             })
-            .join();
+                .join();
         }
     }
 
@@ -664,7 +705,7 @@ mod tests {
                 let res = thread::spawn(move || {
                     rx.recv().unwrap();
                 })
-                .join();
+                    .join();
                 assert!(res.is_err());
             });
             let _t = thread::spawn(move || {
